%%%% Replication codes for Figure C1 in
%%%% "Most powerful test against a sequence of high dimensional local
%%%% alternatives" by Yi He, Sombut Jaidee and Jiti Gao
%%%% Last Update: Feb, 2021

%% Load data: codes from https://research.stlouisfed.org/econ/mccracken/fred-databases/
rng('default')
warning('off')
fredfactors_cut

disp('Data loaded...')
disp('Searching for the column for INDPRO...')
IP_ix=find(contains(series,'INDPRO'));
disp('The column for INDPRO found...')
disp('Preprocessing the raw data...')
y=data(:,6)*100;
X=data;X(:,6)=[];
wd=120;
lag_all=[0;1;2;3;4;5];
N=numel(y);
%% Rolling-window p values
pall=nan(N,numel(lag_all));

bar=1;
disp('Calculating rolling window p-values...')
for i=wd+1:(N-1)
    if i/(N-wd)*10>bar
        fprintf('|');
        bar=bar+1;
    end
    timeix=(i-wd+1):i;
    temp_X=X([timeix-1,i],:);
    X_ix=sum(isnan(X))==0;
    temp_X=temp_X(:,X_ix);
    [temp_X_z,mu_x,sigma_x]=zscore(temp_X(1:end-1,:));
    for lag_ix=1:numel(lag_all)
        lag=lag_all(lag_ix);
        pall(i,lag_ix)=artest(y(timeix),X(timeix-1,:),lag,false,true,false);
    end
end
fprintf('\n')
disp('p-values calculated')
clear bar

%% Export figures for p-values
figC1=figure;
disp('Displaying p-values...')
for lag_ix=1:6
    subplot(3,2,lag_ix)
    area(dates(wd+1:N),pall(wd+1:N,lag_ix),'LineStyle','none')
    axis tight
    ylim([0,1])
    hline = refline(0,0.05);
    hline.Color = 'k';
    hline.LineStyle='--';
end




