function [pval,Q,mdl,sigma_sq] = artest(y,X,lag,iffillna,ifstd,ifAdjust,sigma_sq,ifout_w,ifrobust)

% Last updated: April, 2020
% y: response vector
% X: design matrix of exogenous variables
% lag: order of the autoregressive model (default=0)
% iffillna: TRUE/FALSE(default) = whether to fill in missing values by 0
% ifstd: TRUE/FALSE(default) = whether to standardize the variance of variables
% sigma_sq: regression error variance (default = least-squares estimate)

%% Check arguments and set default values
if nargin<2
    error('Not enough inputs')
elseif nargin<3
    lag=0;
end
if ~exist('iffillna','var')
    iffillna=false;
end
if ~exist('ifstd','var')
    ifstd=false;
end

if ~exist('ifAdjust','var')
    ifAdjust=false;
end

if ~exist('sigma_sq','var')
    sigma_sq=-1; % meaning to be estimated
end
if lag<0
    error('The autoregressive order cannot be negative')
end

if ~exist('ifout_w','var')
    ifout_w=false;
end

if ~exist('ifrobust','var')
    ifrobust=false;
end
%% Calculate the p value
n=numel(y);y=y(:);
Z=nan(n-lag,lag+1);
Z(:,1)=ones(n-lag,1); % intercept
for i=1:lag
    Z(:,i+1)=y((lag-i+1):(n-i));
end
y=y(lag+1:end);
X=X(lag+1:end,:);
if ifstd
    mdl.xstd=nanstd(X);
    X=X./(ones(size(X,1),1)*mdl.xstd);
end
if ifAdjust
    mdl.tv=sqrt(nansum(X.^2,2));
    mdl.tv=mdl.tv/sqrt(sum(mdl.tv.^2)/n);
    X=X./(mdl.tv*ones(1,size(X,2)));
end
if ifout_w
    [pval,Q,sigma_sq,mdl.w,mdl.rhosq]=optimaltest(y,X,Z,iffillna,sigma_sq,ifrobust);
else
    [pval,Q,sigma_sq]=optimaltest(y,X,Z,iffillna,sigma_sq,ifrobust);
end
[n,p]=size(X);
mdl.ord=sqrt(p)/n;
end

