function [tbl1,tbl2,pval_rep] = getTable(N_rep,ord,filename1,filename2,seed)

%%%%
% Generates Tables 1-2 and C1-C2
% N_rep = number of replications
% ord = sqrt(p)/n 
% filename1 = LaTex table filename for uniform stochastic coefficients
% filename2 = LaTex table filename for deterministic coefficients
% seed = random seed number
%%%%

if ~exist('seed','var')
    rng('default');
else
    rng(seed);
end

if ~exist('N_rep','var')
    N_rep=5000;
end


%% Initialize Parameters
r=[1/4,1/2,1,2,4]; % concentration ratio
alp=0.05;
theta=[0;0.3;0.08;0.11]; % intercept=0
arcoeff=0.3;
macoeff=0.3;
N_dgp=4;
N_dir=2;
N_lag=3;
tausq_all=[0;1;2;5]; % Depature levels

%%
n_all=round(r/ord^2);
p_all=round((r/ord).^2);
pval_rep=nan(numel(p_all),numel(tausq_all),N_dir*3,N_dgp,N_rep);
Mdl = arima('Constant',theta(1),'AR',{theta(2),theta(3),theta(4)},'Variance',1);
ARMdl_X=arima('Constant',0,'AR',{arcoeff(1)},'Variance',1);
for j=1:numel(p_all)
    disp(['The case for p/n=',num2str(r(j)),'...'])
    p=p_all(j);n=n_all(j);
    T=toeplitz((0.5).^(0:(p-1)));
    [U,D]=eig(T);D(D(:)<0)=0;
    T_root=U*(sqrt(D))*U';
    % Replace 'for' with 'parfor' below for parallel computing
    for i=1:N_rep
        disp(['p=',num2str(p),' n=',num2str(n),' Rep=',num2str(i)])
        pval_temp=nan(1,numel(tausq_all),N_dir*3,N_dgp,1);
        V=trndstd(5,[n+N_lag+1,p]);
        for ix=1:N_dgp
            switch ix
                case 1
                    X=V(2:end,:);
                case 2
                    X=V(2:end,:)*T_root;
                case 3
                    X=(V(2:end,:)+macoeff(1)*V(1:end-1,:))*T_root;
                otherwise
                    X=filter(ARMdl_X,V(2:end,:))*T_root;
            end
            S=cov(X((end-n+1):end,:));
            w=sqrt(norm(S,'fro')^2/p-p/n*(trace(S)/p)^2);
            for dir_ix=1:N_dir
                switch dir_ix
                    case 1
                        xi=normrnd(0,1,p,1);
                    otherwise
                        xi=U*ones(p,1);
                end
                xi=xi/norm(xi);
                e=trndstd(5,[n+N_lag,1]);
                
                for tau_ix=1:numel(tausq_all)
                    beta=sqrt(tausq_all(tau_ix)*sqrt(p)/n)*xi;
                    Z=X*beta+sqrt(w/sqrt(2))*e;
                    y=filter(Mdl,Z);
                    
                    pval_temp(1,tau_ix,dir_ix,ix,1)=artest(y(end-n+1-N_lag:end),...
                        X(end-n+1-N_lag:end,:),N_lag,false,false,false);
                    
                    pval_temp(1,tau_ix,dir_ix+N_dir,ix,1)= artest(y(end-n+1-N_lag:end),...
                        X(end-n+1-N_lag:end,:),N_lag,false,false,false,w/sqrt(2));
                    
                    pval_temp(1,tau_ix,dir_ix+2*N_dir,ix,1)= artest(y(end-n+1-N_lag:end),...
                        X(end-n+1-N_lag:end,:),N_lag,false,false,false,-1,false,true);
                end
            end
        end
        pval_rep(j,:,:,:,i)=pval_temp;
    end
    fprintf('\n')
end

%% Create the Table column by column
pow_all=round(mean(pval_rep<alp,5)*100,1);
tbl_array=reshape(pow_all,numel(p_all)*numel(tausq_all),N_dir*3*N_dgp);
tbl_array=reshape(string(num2str(tbl_array(:),'%.1f')),size(tbl_array));
%tbl_margin=string(repmat([p_all(:),n_all(:),r(:)],numel(tausq_all),1));
tbl_margin=repmat(string(p_all(:))+'/'+string(n_all(:)),numel(tausq_all),1);
varnames={'p_over_n','IID','IID_oracle','IID_robust',...
    'CSD','CSD_oracle','CSD_robust',...
    'MA1','MA1_oracle','MA1_robust',...
    'AR1','AR1_oracle','AR1_robust',...
    };
tbl1=array2table([tbl_margin,tbl_array(:,1:N_dir:end)],'VariableNames',varnames);
tbl2=array2table([tbl_margin,tbl_array(:,2:N_dir:end)],'VariableNames',varnames);

if exist('filename1','var')
    writetex([tbl_margin,tbl_array(:,1:2:end)],filename1,tausq_all,1);
end

if exist('filename2','var')
    writetex([tbl_margin,tbl_array(:,2:2:end)],filename2,tausq_all,2);
end


end

