function [tbl,pval_rep] = getTableC3(N_rep,filename,seed)

%%%%
% Generate Table C3
% N_rep = number of replications
% filename = LaTex table filename
% seed = random seed number
%%%%

if ~exist('seed','var')
    rng('default');
else
    rng(seed);
end

if ~exist('N_rep','var')
    N_rep=5000;
end


%% Initialize Parameters
alp=0.05;
theta=[0;0.3;0.08;0.11]; % intercept=0
arcoeff=0.3;
macoeff=0.3;
N_dgp=4;
sall=[0,0.5,1,1.5];
N_lag=3;
tausq_all=[0;1;2;5]; % Depature levels
%%
N_dir=numel(sall);
pval_rep=nan(numel(tausq_all),N_dir*3,N_dgp,N_rep);
Mdl = arima('Constant',theta(1),'AR',{theta(2),theta(3),theta(4)},'Variance',1);
ARMdl_X=arima('Constant',0,'AR',{arcoeff(1)},'Variance',1);

p=52;n=294;
T=toeplitz((0.5).^(0:(p-1)));
T_root=sqrtm(T);
for i=1:N_rep
    disp(['p=',num2str(p),' n=',num2str(n),' Rep=',num2str(i)])
    V=trndstd(5,[n+N_lag+1,p]);
    for ix=1:N_dgp
        switch ix
            case 1
                X=V(2:end,:);
            case 2
                X=V(2:end,:)*T_root;
            case 3
                X=(V(2:end,:)+macoeff(1)*V(1:end-1,:))*T_root;
            otherwise
                X=filter(ARMdl_X,V(2:end,:))*T_root;
        end
        S=cov(X((end-n+1):end,:));
        [U,D]=eig(S);D(D(:)<0)=0;lam=diag(D);
        for dir_ix=1:N_dir
            s=sall(dir_ix);
            xi=U*(D.^(s/2))*ones(p,1);
            xi=xi/norm(xi);
            w=sum(lam.^(s+2))/sum(lam.^s)...
                -p/n*sum(lam.^(s+1))/sum(lam.^s)*mean(lam);
            w=w/sqrt(norm(S,'fro')^2/p-p/n*(trace(S)/p)^2);
            e=trndstd(5,[n+N_lag,1]);
   
            for tau_ix=1:numel(tausq_all)
                beta=sqrt(tausq_all(tau_ix)*sqrt(p)/n)*xi;
                Z=X*beta+sqrt(w/sqrt(2))*e;
                y=filter(Mdl,Z);
                
                pval_rep(tau_ix,dir_ix,ix,i)= artest(y(end-n+1-N_lag:end),...
                    X(end-n+1-N_lag:end,:),N_lag,false,false,false);
                
                pval_rep(tau_ix,dir_ix+N_dir,ix,i)= artest(y(end-n+1-N_lag:end),...
                    X(end-n+1-N_lag:end,:),N_lag,false,false,false,w/sqrt(2));
                
                pval_rep(tau_ix,dir_ix+2*N_dir,ix,i)= artest(y(end-n+1-N_lag:end),...
                    X(end-n+1-N_lag:end,:),N_lag,false,false,false,-1,false,true);
            end
        end
    end
end
fprintf('\n');

%% Create the Table column by column
pow_all=round(mean(pval_rep<alp,4)*100,1);
tbl_margin=repmat(string(sall(:)),numel(tausq_all),1);
varnames={'s','IID','IID_oracle','IID_robust',...
    'CSD','CSD_oracle','CSD_robust',...
    'MA1','MA1_oracle','MA1_robust',...
    'AR1','AR1_oracle','AR1_robust',...
    };
tbl_full=[];
 for i=1:numel(tausq_all)
     tbl_full=[tbl_full;reshape(pow_all(i,:,:),4,12)];
 end
tbl_full=reshape(string(num2str(tbl_full(:),'%.1f')),size(tbl_full));

tbl=array2table([tbl_margin,tbl_full],'VariableNames',varnames);

if exist('filename','var')
    writetexC3(tbl,filename,tausq_all);
end


end

