function [tbl1,tbl2,pval_rep] = getTableC4(N_rep,filename1,filename2,seed)

%%%%
% Generate Table C4
% tbl1 = the output for using the true variance (not reported)
% tbl2 = Table C4
% filename1 = LaTex table filename for tbl1
% filename2 = LaTex table filename for tbl2
% seed = random seed number
%%%%

if ~exist('seed','var')
    rng('default');
else
    rng(seed);
end

if ~exist('N_rep','var')
    N_rep=5000;
end


%% unpack the option structure
r=[1/4,1/2,1,2,4]; % concentration ratio
ord=0.05;
alp=0.05;
arcoeff=0.3;
macoeff=0.3;
N_dgp=4;
N_dir=2;
N_lag=1;
tausq_all=[0;1;2;5]; % Depature levels
rho=[0,0.2,-0.2];
psi_1=0.3;
var_z=1;
%%
n_all=round(r/ord^2);
p_all=round((r/ord).^2);
pval_rep=nan(numel(p_all),numel(tausq_all),N_dir*3,N_dgp,N_rep);
ARMdl_X=arima('Constant',0,'AR',{arcoeff(1)},'Variance',1);
for j=1:numel(p_all)
    disp(['The case for p/n=',num2str(r(j)),'...'])
    p=p_all(j);n=n_all(j);
    T=toeplitz((0.5).^(0:(p-1)));
    [U,D]=eig(T);D(D(:)<0)=0;
    T_root=U*(sqrt(D))*U';
    % Replace 'for' with 'parfor' below for parallel computing
    for i=1:N_rep
        disp(['p=',num2str(p),' n=',num2str(n),' Rep=',num2str(i)])
        pval_temp=nan(1,numel(tausq_all),N_dir*3,N_dgp,1);
        V=trndstd(5,[n+N_lag+1,p]);
        for ix=1:N_dgp
            switch ix
                case 1
                    X=V(2:end,:);
                    agg_lagcorr=0;
                case 2
                    X=V(2:end,:)*T_root;
                    agg_lagcorr=0;
                case 3
                    X=(V(2:end,:)+macoeff(1)*V(1:end-1,:))*T_root;
                    agg_lagcorr=0.3/sqrt(0.3^2+1);
                otherwise
                    X=filter(ARMdl_X,V(2:end,:))*T_root;
                    agg_lagcorr=0.3;
            end
            S=cov(X((end-n+1):end,:));
            w=sqrt(norm(S,'fro')^2/p-p/n*(trace(S)/p)^2);
            sig_err=sqrt(w/sqrt(2));           
            for dir_ix=1:N_dir
                switch dir_ix
                    case 1
                        sigma_sq=sig_err^2;
                    otherwise
                        sigma_sq=-1;
                end
                xi=normrnd(0,1,p,1);
                xi=xi/norm(xi);
                e=trndstd(5,[n+N_lag,1]);
                v=normrnd(0,1,n+N_lag,1);
                
                for tau_ix=1:numel(tausq_all)
                    beta=sqrt(tausq_all(tau_ix)*sqrt(p)/n)*xi;
                    X_proj=X*beta;
                    for psi_ix=1:3
                    E_init=X_proj+sig_err*e;
                    if tausq_all(tau_ix)>0
                        psi_0=rho(psi_ix)*sqrt(var_z/var(X_proj))-psi_1*agg_lagcorr;
                    else
                        psi_0=0;
                    end
                    Z=[nan;E_init(1:end-1)]*psi_1+psi_0*X_proj;
                    sig_v_sq=var_z...
                        -(tausq_all(tau_ix)*sqrt(p)/n)^2*var(X*xi)...
                        *(psi_1^2+psi_0^2+2*psi_1*psi_0*agg_lagcorr)...
                        -psi_1^2*sig_err^2;
                    Z=Z+sqrt(sig_v_sq)*v;
                    
                    y=E_init+Z;
                    
                    pval_temp(1,tau_ix,dir_ix+(psi_ix-1)*N_dir,ix,1)=...
                        optimaltest(y((N_lag+1):end),X((N_lag+1):end,:),Z((N_lag+1):end),...
                        false,sigma_sq,true);
                    end
                end
            end
        end
        pval_rep(j,:,:,:,i)=pval_temp;
    end
    fprintf('\n')
end

%% Create the Table column by column
pow_all=round(mean(pval_rep<alp,5)*100,1);
tbl_array=reshape(pow_all,numel(p_all)*numel(tausq_all),N_dir*3*N_dgp);
tbl_array=reshape(string(num2str(tbl_array(:),'%.1f')),size(tbl_array));
tbl_margin=repmat(string(p_all(:))+'/'+string(n_all(:)),numel(tausq_all),1);
varnames={'p_over_n','IID','IID_Pos','IID_Neg',...
    'CSD','CSD_Pos','CSD_Neg',...
    'MA1','MA1_Pos','MA1_Neg',...
    'AR1','AR1_Pos','AR1_Neg',...
    };
tbl1=array2table([tbl_margin,tbl_array(:,1:N_dir:end)],'VariableNames',varnames);
tbl2=array2table([tbl_margin,tbl_array(:,2:N_dir:end)],'VariableNames',varnames);

if exist('filename1','var') && ~isempty(filename1)
    writetexC4([tbl_margin,tbl_array(:,1:2:end)],filename1,tausq_all,1);
end

if exist('filename2','var')
    writetexC4([tbl_margin,tbl_array(:,2:2:end)],filename2,tausq_all,2);
end


end

