function [pval,Q,sigma_sq,w,rhosq] = optimaltest(y,X,Z,iffillna,sigma_sq,ifrobust)

% Last updated: April, 2020
% y: response vector
% X: design matrix of exogenous variables
% Z: nuisance variables
% iffillna: TRUE/FALSE(default) = whether to fill in missing values by 0
% sigma_sq: regression error variance (default = least-squares estimate)
% ifrobust: use the robust test or not

%% Check arguments and set default values
if nargin<2
    error('Not enough input arguments')
end
if ~exist('Z','var')
    Z=ones(numel(y),1);
end
if ~exist('iffillna','var')
    iffillna=false;
end
if ~exist('sigma_sq','var')
    sigma_sq=-1; % meaning to be estimated
end
if ~exist('ifrobust','var')
    ifrobust=false; 
end

%% Calculate p value

y=y(:);n=numel(y); d=size(Z,2); 
X=X-ones(n,1)*nanmean(X);
if iffillna
    X_ix=sum(~isnan(X))>=12;
    X=X(:,X_ix);
    wgt=n./(sum(~isnan(X)));
    X=fillmissing(X,'constant',0);
    X=X*diag(sqrt(wgt));
else
    X_ix=sum(isnan(X))==0;
    X=X(:,X_ix);
end
[~,~,e] = regress(y,Z);

if sigma_sq<=0
    sigma_sq=e'*e/(n-d);
end
if sigma_sq==0
    error('The least square residuals are all zeos')
end
e=e/sqrt(sigma_sq);
A=(X*X')/n;
if nargout>3
   p=size(X,2);
   w=sqrt(var(eig(A),1)*n/p);
end

A=A-diag(diag(A)); A=tril(A);
Q=e'*A*e/(norm(A,'fro'));
if ifrobust
    P_Z=Z/(Z'*Z)*Z';
    v=A*e;
    rhosq=v'*P_Z*v/(v'*v);
    Q=Q/sqrt(1-rhosq);
else
    rhosq=0;
end
pval=1-normcdf(Q);


end

